@@ -67,7 +67,7 @@ def upgrade():
             discriminator = 'RelativeModifier'
             relatives.append({
                     'id': modifier_id,
-                    'value': modifier_value,
+                    'value': modifier_value / 100,
             })
         update_stmt = update(modifier)\
                 .where(modifier.c.id == modifier_id)\
@@ -91,7 +91,9 @@ def upgrade():
 
 def downgrade():
     # Add type_ and value columns back
-    op.add_column('modifier', sa.Column('type_', sa.Enum('absolute', 'percentage', name='modifier_type'), nullable=False))
+    op.add_column('modifier',
+            sa.Column('type_', sa.Enum('absolute', 'percentage',
+                    name='modifier_type')))
     op.add_column('modifier', sa.Column('value', sa.Float, nullable=True))
     # populate type_ and value columns with data from the subclass tables
     abs_select = select([abs_table.c.id, abs_table.c.value])
@@ -117,3 +119,9 @@ def downgrade():
     op.drop_column('modifier', '_type')
     op.drop_table('absolute_modifier')
     op.drop_table('relative_modifier')
+    # Add not-null constraint back to type_
+    op.alter_column('modifier',
+            column_name='type_',
+            nullable=False,
+            existing_type=sa.Enum('absolute', 'percentage',
+                    name='modifier_type'))
