@@ -13,9 +13,11 @@ from wtforms.validators import InputRequired, AnyOf, URL, ValidationError,\
         StopValidation
 
 from .. import db
-from ..models import Request, Modifier, Action
-from ..auth import SubmitRequestsPermission, ReviewRequestsPermission, \
-        PayoutRequestsPermission, admin_permission
+from ..models import Request, Modifier, Action, ActionType, ActionError,\
+        ModifierError, AbsoluteModifier, RelativeModifier
+from ..auth.permissions import SubmitRequestsPermission,\
+        ReviewRequestsPermission, PayoutRequestsPermission, admin_permission
+from ..auth import PermissionType
 from ..auth.models import Division, Pilot, Permission, User, Group, Note
 
 
@@ -104,6 +106,13 @@ class PermissionRequestListing(RequestListing):
         self.permissions = permissions
         self.statuses = statuses
 
+    def dispatch_request(self, division_id=None, page=1):
+        if not current_user.has_permission(self.permissions):
+            abort(403)
+        else:
+            return super(PermissionRequestListing, self).dispatch_request(
+                    division_id, page)
+
     def requests(self, division_id=None):
         user_perms = db.session.query(Permission.id.label('permission_id'),
                 Permission.division_id.label('division_id'),
@@ -133,13 +142,16 @@ class PayoutListing(PermissionRequestListing):
 
     def __init__(self):
         # Just a special case of PermissionRequestListing
-        super(PayoutListing, self).__init__(('pay',), ('approved',))
+        super(PayoutListing, self).__init__((PermissionType.pay,),
+                (ActionType.approved,))
 
     def dispatch_request(self, division_id=None, page=1):
         """Returns the response to requests.
 
         Part of the :py:class:`flask.views.View` interface.
         """
+        if not current_user.has_permission(self.permissions):
+            abort(403)
         pager = self.requests(division_id).paginate(page, per_page=20)
         return render_template(self.template, form=ActionForm(), pager=pager)
 
@@ -154,7 +166,7 @@ def register_perm_request_listing(app, endpoint, path, permissions, statuses):
     :param str path: The URL path for the view
     :param tuple permissions: Passed to
         :py:meth:`PermissionRequestListing.__init__`
-    :param callable statuses: Passed to
+    :param iterable statuses: Passed to
         :py:meth:`PermissionRequestListing.__init__`
     """
     view = PermissionRequestListing.as_view(endpoint, permissions=permissions,
@@ -186,9 +198,9 @@ def register_class_views(state):
     state.add_url_rule(payout_url_stub + '<int:page>/<int:division_id>/',
             view_func=payout_view)
     register_perm_request_listing(state, 'list_pending_requests',
-            '/pending/', ('review',), ('evaluating', 'incomplete', 'approved'))
+            '/pending/', (PermissionType.review,), ActionType.pending)
     register_perm_request_listing(state, 'list_completed_requests',
-            '/completed/', ('review', 'pay'), ('rejected', 'paid'))
+            '/completed/', PermissionType.elevated, ActionType.finalized)
 
 
 class ValidKillmail(URL):
@@ -258,7 +270,7 @@ def submit_divisions(user):
     :rtype: list
     """
     submit_perms = user.permissions\
-            .filter_by(permission='submit')\
+            .filter_by(permission=PermissionType.submit)\
             .subquery()
     divisions = db.session.query(Division).join(submit_perms)\
             .order_by(Division.name)
@@ -280,7 +292,7 @@ def submit_request():
     valid. Also enforces that the user submitting this requests controls the
     character from the killmail and prevents duplicate requests.
     """
-    if not current_user.has_permission('submit'):
+    if not current_user.has_permission(PermissionType.submit):
         abort(403)
     form = RequestForm()
     # Create a list of divisions this user can submit to
@@ -341,8 +353,8 @@ class PayoutForm(Form):
 class ActionForm(Form):
     id_ = HiddenField(default='action')
     note = TextAreaField('Note')
-    type_ = HiddenField(default='comment', validators=[AnyOf(('rejected',
-            'evaluating', 'approved', 'incomplete', 'paid', 'comment'))])
+    type_ = HiddenField(default='comment',
+            validators=[AnyOf(ActionType.values())])
 
 
 @blueprint.route('/<int:request_id>/', methods=['GET', 'POST'])
@@ -358,43 +370,34 @@ def request_detail(request_id):
 
     :param int request_id: the ID of the request
 
-    .. digraph:: request_workflow
-
-        rankdir="LR";
-
-        sub [label="submitted", shape=plaintext];
-
-        node [style="dashed, filled"];
-
-        eval [label="evaluating", fillcolor="#fcf8e3"];
-        rej [label="rejected", style="solid, filled", fillcolor="#f2dede"];
-        app [label="approved", fillcolor="#d9edf7"];
-        inc [label="incomplete", fillcolor="#f2dede"];
-        paid [label="paid", style="solid, filled", fillcolor="#dff0d8"];
-
-        sub -> eval;
-        eval -> rej [label="R"];
-        eval -> app [label="R"];
-        eval -> inc [label="R"];
-        rej -> eval [label="R"];
-        inc -> eval [label="R, S"];
-        inc -> rej [label="R"];
-        app -> paid [label="P"];
-        app -> eval [label="R"];
-        paid -> eval [label="P"];
-        paid -> app [label="P"];
-
-    R means a reviewer can make that change, S means the submitter can make
-    that change, and P means payers can make that change. Solid borders are
-    terminal states.
     """
-    srp_request = Request.query.get(request_id)
+    def render_details():
+        # Different templates are used for different roles
+        if review_perm.can():
+            template = 'request_review.html'
+        elif pay_perm.can():
+            template = 'request_detail.html'
+        elif current_user == srp_request.submitter:
+            template = 'request_detail.html'
+        else:
+            abort(403)
+        return render_template(template, srp_request=srp_request,
+                modifier_form=ModifierForm(formdata=None),
+                payout_form=PayoutForm(formdata=None),
+                action_form=ActionForm(formdata=None),
+                void_form=VoidModifierForm(formdata=None))
+
+    srp_request = Request.query.get_or_404(request_id)
     review_perm = ReviewRequestsPermission(srp_request)
     pay_perm = PayoutRequestsPermission(srp_request)
     if request.method == 'POST':
+        failed = False
         if request.form['id_'] == 'modifier':
             form = ModifierForm()
         elif request.form['id_'] == 'payout':
+            if not review_perm.can():
+                flash("Only reviewers can change the base payout.", 'error')
+                return render_details()
             form = PayoutForm()
         elif request.form['id_'] == 'action':
             form = ActionForm()
@@ -403,112 +406,68 @@ def request_detail(request_id):
         else:
             abort(400)
         if form.validate():
-            if srp_request.status == 'evaluating':
-                if form.id_.data == 'modifier':
-                    if review_perm.can():
-                        mod = Modifier(srp_request, current_user, form.note.data)
-                        if form.type_.data == 'rel-bonus':
-                            mod.type_ = 'percentage'
-                            mod.value = form.value.data
-                        elif form.type_.data == 'rel-deduct':
-                            mod.type_ = 'percentage'
-                            mod.value = form.value.data * -1
-                        elif form.type_.data == 'abs-bonus':
-                            mod.type_ = 'absolute'
-                            mod.value = form.value.data
-                        elif form.type_.data == 'abs-deduct':
-                            mod.type_ = 'absolute'
-                            mod.value = form.value.data * -1
-                        db.session.add(mod)
-                        db.session.commit()
-                    else:
-                        flash("Insufficient permissions.", 'error')
-                elif form.id_.data == 'payout':
-                    if review_perm.can():
-                        srp_request.base_payout = form.value.data
-                        db.session.commit()
-                    else:
-                        flash("Insufficient permissions.", 'error')
-                elif form.id_.data == 'void':
-                    if review_perm.can():
-                        modifier = Modifier.query.get(
-                                int(form.modifier_id.data))
-                        modifier.void(current_user)
-                        db.session.commit()
-                    else:
-                        flash("Insufficient permissions.", 'error')
-            if form.id_.data == 'action':
-                # For serious, look at the diagram in the documentation before
-                # tinkering around in here.
-                type_ = form.type_.data
-                invalid = False
-                if srp_request.status == 'evaluating':
-                    if type_ not in ('approved', 'rejected', 'incomplete',
-                            'comment'):
-                        flash("Cannot go from Evaluating to Paid", 'error')
-                        invalid = True
-                    elif type_ != 'comment' and not review_perm.can():
-                        flash("You are not a reviewer.", 'error')
-                        invalid = True
-                elif srp_request.status == 'incomplete':
-                    if type_ not in ('evaluating', 'rejected', 'comment'):
-                        flash("Can only reject or re-evaluate.", 'error')
-                        invalid = True
-                    elif type_ == 'evaluating' and not (review_perm.can() or
-                            srp_request.submitter == current_user):
-                        flash(("You must be a reviewer or own this request to"
-                               "re-evaluate."), 'error')
-                        invalid = True
-                    elif type_ == 'rejected' and not review_perm.can():
-                        flash("You are not a reviewer.", 'error')
-                        invalid = True
-                elif srp_request.status == 'rejected':
-                    if type_ not in ('evaluating', 'comment'):
-                        flash("Can only change to Evaluating.", 'error')
-                        invalid = True
-                    elif type_ != 'comment' and not review_perm.can():
-                        flash("You are not a reviewer.", 'error')
-                        invalid = True
-                elif srp_request.status == 'approved':
-                    if type_ not in ('paid', 'evaluating', 'comment'):
-                        flash("Can only set to Evaluating or Paid.", 'error')
-                        invalid = True
-                    elif type_ == 'paid' and not pay_perm.can():
-                        flash("You are not a payer.", 'error')
-                        invalid = True
-                    elif type_ == 'evaluating' and not review_perm.can():
-                        flash("You are not a reviewer.", 'error')
-                        invalid = True
-                elif srp_request.status == 'paid':
-                    if type_ not in ('comment', 'approved', 'evaluating'):
-                        flash("""Can only move to Approved or Evaluating from
-                                Paid.""", 'error')
-                        invalid = True
-                    elif type_ != 'comment' and not pay_perm.can():
-                        flash("You are not a payer.", 'error')
-                        invalid = True
-                if not invalid:
-                    action = Action(srp_request, current_user,
-                            form.note.data)
-                    action.type_ = type_
+            if form.id_.data == 'modifier':
+                if 'bonus' in form.type_.data:
+                    value = form.value.data
+                elif 'deduct' in form.type_.data:
+                    vaue = form.value.data * -1
+                if 'abs' in form.type_.data:
+                    ModClass = AbsoluteModifier
+                elif 'rel' in form.type_.data:
+                    ModClass = RelativeModifier
+                try:
+                    mod = ModClass(srp_request, current_user, form.note.data,
+                            value)
+                except ModifierError as e:
+                    flash(e, 'error')
+                    return render_details()
+                if form.type_.data == 'rel-bonus':
+                    mod.type_ = 'percentage'
+                    mod.value = form.value.data
+                elif form.type_.data == 'rel-deduct':
+                    mod.type_ = 'percentage'
+                    mod.value = form.value.data * -1
+                elif form.type_.data == 'abs-bonus':
+                    mod.type_ = 'absolute'
+                    mod.value = form.value.data
+                elif form.type_.data == 'abs-deduct':
+                    mod.type_ = 'absolute'
+                    mod.value = form.value.data * -1
+                db.session.add(mod)
+                db.session.commit()
+            elif form.id_.data == 'payout':
+                try:
+                    srp_request.base_payout = form.value.data
                     db.session.commit()
+                except ModifierError as e:
+                    flash(e, 'error')
+                    return render_details()
+            elif form.id_.data == 'void':
+                modifier_id = int(form.modifier_id.data)
+                modifier = Modifier.query.get(modifier_id)
+                if modifier is None:
+                    flash("Invalid modifier ID {}.".format(modifier_id),
+                            'error')
+                    return render_details()
+                try:
+                    modifier.void(current_user)
+                    db.session.commit()
+                except ModifierError as e:
+                    flash(e, 'error')
+                    return render_details()
+            elif form.id_.data == 'action':
+                type_ = ActionType.from_string(form.type_.data)
+                try:
+                    Action(srp_request, current_user, form.note.data, type_)
+                    db.session.commit()
+                except ActionError as e:
+                    flash(e, 'error')
+                    return render_details()
         else:
             # TODO: Actual error handling, probably using flash()
             print(form.errors)
     # Different templates are used for different roles
-    if review_perm.can():
-        template = 'request_review.html'
-    elif pay_perm.can():
-        template = 'request_detail.html'
-    elif current_user == srp_request.submitter:
-        template = 'request_detail.html'
-    else:
-        abort(403)
-    return render_template(template, srp_request=srp_request,
-            modifier_form=ModifierForm(formdata=None),
-            payout_form=PayoutForm(formdata=None),
-            action_form=ActionForm(formdata=None),
-            void_form=VoidModifierForm(formdata=None))
+    return render_details()
 
 
 class DetailForm(Form):
@@ -527,7 +486,7 @@ def request_detail_update(request_id):
     if form.validate_on_submit():
         archive_note = 'Old Details: ' + srp_request.details
         archive_action = Action(srp_request, current_user, archive_note)
-        archive_action.type_ = 'evaluating'
+        archive_action.type_ = ActionType.evaluating
         srp_request.details = form.details.data
         db.session.commit()
         return redirect(url_for('.request_detail', request_id=request_id))
@@ -557,10 +516,16 @@ def request_change_division(request_id):
                 srp_request.division.name,
                 new_division.name)
         archive_action = Action(srp_request, current_user, archive_note)
-        archive_action.type_ = 'evaluating'
+        archive_action.type_ = ActionType.evaluating
         srp_request.division = new_division
         db.session.commit()
-        return redirect(url_for('.request_detail', request_id=request_id))
+        flash('Request #{} moved to {} division'.format(srp_request.id,
+                new_division.name), 'success')
+        if current_user.has_permission(PermissionType.elevated, new_division) or\
+                current_user == srp_request.submitter:
+            return redirect(url_for('.request_detail', request_id=request_id))
+        else:
+            return redirect(url_for('.list_pending_requests'))
     form.division.data = srp_request.division.id
     return render_template('form.html', form=form)
 
@@ -568,7 +533,7 @@ def request_change_division(request_id):
 @blueprint.route('/notes/<int:user_id>/', methods=['GET', 'POST'])
 @login_required
 def user_notes(user_id):
-    if not current_user.has_permission(('review', 'pay')):
+    if not current_user.has_permission(PermissionType.elevated):
         abort(403)
     user = User.query.get_or_404(user_id)
     return render_template('notes.html', user=user)
@@ -578,7 +543,9 @@ class AddNote(Form):
     note = TextAreaField('Note',
             description=("If you have something like '#{Kill ID}', it will be "
                          "linkified to the corresponding request "
-                         "(if it exists)."),
+                         "(if it exists). For example, #1234567 would be "
+                         "linked to the request for the kill with ID "
+                         "1234567."),
             validators=[InputRequired()])
     submit = SubmitField('Submit')
 
@@ -589,7 +556,7 @@ killmail_re = re.compile(r'#(\d+)')
 @blueprint.route('/notes/<int:user_id>/add/', methods=['GET', 'POST'])
 @login_required
 def add_user_note(user_id):
-    if not current_user.has_permission(('review', 'pay')):
+    if not current_user.has_permission(PermissionType.elevated):
         abort(403)
     user = User.query.get_or_404(user_id)
     form = AddNote()
